"""
Functions and routines associated with Enasis Network Remote Connect.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Any



def dumlog(
    **kwargs: Any,
) -> None:
    """
    Capture and drop messages for would be provided logger.

    :param kwargs: Keyword arguments which would be passed.
    """
